import rs1101.random_string as rs
import rs1101.random_format as rf
import click
import cloup


class Repo:
    pass


default_length = 10


@click.group(invoke_without_command=True)
@cloup.option_group(
    "candidate options",
    click.option(
        "-c",
        metavar="canddidate",
        type=click.Choice(list(rs.candidate_dict.keys())),
        multiple=True,
        default=rs.cddt_default,
    ),
    click.option("-cs", type=str, default=""),
    constraint=cloup.constraints.mutually_exclusive,
)
@click.option(
    "-l", metavar="the length of the generated string", type=int, default=None
)
@click.option(
    "-ss",
    metavar="show strength",
    is_flag=True,
    show_default=True,
    default=False,
)
@click.option(
    "-s",
    metavar="secret string",
    type=str,
    default=None,
)
@click.pass_context
def cli(ctx, c, cs, l, ss, s):
    candidate = rs.g_candidate(c)
    if cs:
        candidate = cs
    ctx.obj = Repo()
    ctx.obj.candidate = candidate
    ctx.obj.length = l if l else default_length
    ctx.obj.ss = ss
    if ctx.invoked_subcommand is None:
        if l is None:
            l = default_length
        if s is not None:
            result = rs.s2rs(s, l, candidate)
        else:
            result = rs.random_string(l, candidate)
        print(result)
        show_strength(ctx.obj)
    else:
        if l is not None:
            raise click.BadOptionUsage(
                message="You should not use the -l option here when using the subcommand."
            )


@cli.command("rs2int")
@click.argument("s", metavar="a string", type=str)
@click.pass_obj
def cli_rs2int(obj, s):
    x = rs.rs2int(s, candidate=obj.candidate)
    obj.length = len(s)
    print(x)
    show_strength(obj)


@cli.command("int2rs")
@click.argument("x", metavar="a int", type=int)
@click.option(
    "-l", metavar="the length of the generated string", type=int, default=None
)
@click.pass_obj
def cli_int2rs(obj, x, l):
    s = rs.int2rs(x, length=l, candidate=obj.candidate)
    obj.length = len(s)
    print(s)
    show_strength(obj)


@cli.command("gen")
@click.option(
    "-l", metavar="the length of the generated string", type=int, default=default_length
)
@click.pass_obj
def cli_rs(obj, l):
    s = rs.random_string(l, candidate=obj.candidate)
    obj.length = l
    print(s)
    show_strength(obj)


def show_strength(obj):
    if obj.ss:
        print(f"strength:{rs.strength(obj.length,len(obj.candidate))}")


@cli.command("mac")
def rsmac():
    print(rf.random_mac())


if __name__ == "__main__":
    cli()



import os


def bytes_length(n):
    return (len(bin(n)) - 2 + 7) // 8


def random_int(n):
    ret = n
    while ret >= n:
        l = bytes_length(n)
        bs = os.urandom(l)
        ret = int.from_bytes(bs, byteorder="big")
    return ret


def choice(candidate):
    l = len(candidate)
    return candidate[random_int(l)]


if __name__ == "__main__":
    print(random_int(512))



from rs1101 import random_string as rs


def random_mac():
    s = rs.random_string(12, rs.candidate_dict["h"])
    lst = [s[i : i + 2] for i in range(0, len(s), 2)]
    ret = ":".join(lst)
    return ret


if __name__ == "__main__":
    print(random_mac())



from hashlib import sha256
from math import inf, log
from string import (
    ascii_letters,
    ascii_lowercase,
    ascii_uppercase,
    digits,
    printable,
    punctuation,
)
from typing import List
import rs1101.random_int as ri


hexdigits = digits + ascii_letters[:6]
Hexdigits = hexdigits.upper()
candidate_dict = {
    "u": ascii_uppercase,
    "l": ascii_lowercase,
    "d": digits,
    "p": punctuation,
    "h": hexdigits,
    "H": Hexdigits,
    "a": printable,
}
cddt_default = ["u", "l", "d"]

candidate = ""
for x in cddt_default:
    candidate += candidate_dict[x]


def strength(length, clen):
    return int(log(clen**length, 2))


def random_string(length, candidate=candidate):
    ret = [ri.choice(candidate) for _ in range(length)]
    return "".join(ret)


def s2rs(s, length, candidate=candidate):
    x = sha256(s.encode("utf8")).hexdigest()
    x = int(x, 16)
    ret = int2rs(x, length, candidate)
    return ret


def wash_cddt(cddt: List[str]):
    cddt = sorted(set(cddt))
    exclusive = sorted(["a", "H", "h"])
    for x in exclusive:
        if x in cddt:
            cddt = [x]
            break
    return cddt


def g_candidate(cddt):
    cddt = wash_cddt(cddt)
    candidate_lst = []
    for x in cddt:
        candidate_lst.append(candidate_dict[x])
    candidate = "".join(candidate_lst)
    return candidate


def int2rs(x, length=None, candidate=candidate):
    l = len(candidate)
    ret = []
    remain = length if length else inf
    while x > 0 and remain > 0:
        x, r = divmod(x, l)
        ret.append(candidate[r])
        remain -= 1
    if length and len(ret) < length:
        lack = length - len(ret)
        ret.append(candidate[0] * lack)
    return "".join(ret[::-1])


def rs2int(rs, candidate=candidate):
    l = len(candidate)
    ret = 0
    weight = 1
    for x in rs[::-1]:
        ret += candidate.index(x) * weight
        weight *= l
    return ret


if __name__ == "__main__":
    length = 20
    s = random_string(length)
    strength = strength(length, len(candidate))
    print(s, strength)
    print(hexdigits)
    x = rs2int(s)
    y = int2rs(x)
    assert s == y
    print(s, x, y)
    print(s2rs("嗨", 10))
    print(s2rs("嗨", 20))
